package com.perforce.maven.scm.provider.p4.command.info;

import java.io.File;

import org.apache.maven.scm.CommandParameter;
import org.apache.maven.scm.CommandParameters;
import org.apache.maven.scm.ScmFileSet;
import org.apache.maven.scm.ScmTag;
import org.apache.maven.scm.ScmTckTestCase;
import org.apache.maven.scm.command.info.InfoItem;
import org.apache.maven.scm.command.info.InfoScmResult;
import org.apache.maven.scm.provider.ScmProvider;
import org.codehaus.plexus.util.FileUtils;
import org.junit.Assert;

import com.perforce.maven.scm.provider.p4.P4ScmTestUtils;

public class P4InfoCommandTckTest
    extends ScmTckTestCase // There is no Info command TCK test, so we write our own
{

    /** {@inheritDoc} */
    public String getScmUrl()
        throws Exception
    {
        return P4ScmTestUtils.getScmUrl();
    }

    /** {@inheritDoc} */
    public void initRepo()
        throws Exception
    {
        P4ScmTestUtils.initRepo();
        resetToOriginalFiles();
    }

    private long initialRevision = 0;

    private void resetToOriginalFiles()
        throws Exception
    {
        checkOut( getWorkingCopy(), getScmRepository() );
        checkOut( getAssertionCopy(), getScmRepository() );

        initialRevision = this.getLastRevisionFromLastSync( null, getWorkingCopy() );

        ScmFileSet fileSet = new ScmFileSet( getWorkingCopy(), P4ScmTestUtils.getAddedIncludes(), null );
        getScmManager().getProviderByUrl( getScmUrl() ).remove( getScmRepository(), fileSet,
                                                                "Remove previous test files" );
        fileSet = new ScmFileSet( getWorkingDirectory(), P4ScmTestUtils.getTestIncludes(), null );
        getScmManager().getProviderByUrl( getScmUrl() ).edit( getScmRepository(), fileSet );
        P4ScmTestUtils.populateOriginalData( getBasedir(), getWorkingCopy() );
        checkIn( getWorkingCopy(), getScmRepository() );
        FileUtils.deleteDirectory( getWorkingDirectory() );
    }

    @SuppressWarnings( "deprecation" )
    public void testInfoCommand()
        throws Exception
    {
        long latestRevision = this.getLastRevisionFromLastSync( null, getWorkingCopy() );
        Assert.assertEquals( initialRevision + 1, latestRevision );

        String tag = getTagName();

        getScmManager().getProviderByUrl( getScmUrl() ).tag( getScmRepository(), new ScmFileSet( getWorkingCopy() ),
                                                             tag );

        Assert.assertEquals( latestRevision, getLastRevisionFromLastSync( tag, getWorkingCopy() ) );

        Assert.assertEquals( initialRevision, getLastRevisionFromLastSync( null, getAssertionCopy() ) );

        Assert.assertEquals( latestRevision, getLastRevisionFromRepo( tag, getAssertionCopy() ) );

        FileUtils.deleteDirectory( getAssertionCopy() );
    }

    private long getLastRevisionFromLastSync( String tagName, File baseDir )
        throws Exception
    {
        return Long.parseLong( getInfo( tagName, baseDir ).getLastChangedRevision() );
    }

    private long getLastRevisionFromRepo( String tagName, File baseDir )
        throws Exception
    {
        return Long.parseLong( getInfo( tagName, baseDir ).getRevision() );
    }

    private InfoItem getInfo( String tagName, File baseDir )
        throws Exception
    {
        CommandParameters parameters = null;

        if ( tagName != null )
        {
            parameters = new CommandParameters();
            parameters.setScmVersion( CommandParameter.SCM_VERSION, new ScmTag( tagName ) );
        }

        ScmFileSet fileSet = new ScmFileSet( baseDir );
        ScmProvider provider = getScmManager().getProviderByUrl( getScmUrl() );
        InfoScmResult result = provider.info( getScmRepository().getProviderRepository(), fileSet, parameters );
        return result.getInfoItems().get( 0 );

    }

    private String getTagName()
    {
        return "groupId:artifactId:tck-info-" + System.currentTimeMillis();
    }
}
